@@ -4,7 +4,7 @@
 ###
 
 setClass("GAlignmentsList",
-    contains="CompressedList",
+    contains="CompressedRangesList",
     representation(
         unlistData="GAlignments",
         elementMetadata="DataFrame"
@@ -84,18 +84,6 @@ setMethod("njunc", "GAlignmentsList",
     function(x) relist(njunc(unlist(x, use.names=FALSE)), x)
 )
 
-setMethod("start", "GAlignmentsList",
-    function(x, ...) relist(start(unlist(x, use.names=FALSE), ...), x)
-)
-
-setMethod("end", "GAlignmentsList",
-    function(x, ...) relist(end(unlist(x, use.names=FALSE), ...), x)
-)
-
-setMethod("width", "GAlignmentsList",
-    function(x) relist(width(unlist(x, use.names=FALSE)), x)
-)
-
 setMethod("seqinfo", "GAlignmentsList",
     function(x) seqinfo(unlist(x, use.names=FALSE))
 )
@@ -104,6 +92,7 @@ setMethod("elementMetadata", "GAlignmentsList",
     GenomicRanges:::get_GRangesList_mcols
 )
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Setters.
 ###
@@ -137,6 +126,7 @@ setReplaceMethod("seqnames", "GAlignmentsList",
     GenomicRanges:::set_GRangesList_seqnames
 )
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
 ###
@@ -149,6 +139,7 @@ setReplaceMethod("seqnames", "GAlignmentsList",
 
 setValidity2("GAlignmentsList", .valid.GAlignmentsList)
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructors.
 ###
@@ -168,20 +159,6 @@ GAlignmentsList <- function(...)
     relist(unlistData, PartitioningByEnd(listData))
 }
 
-setMethod("updateObject", "GAlignmentsList",
-    function(object, ..., verbose=FALSE)
-    {
-        if (verbose)
-            message("updateObject(object = 'GAlignmentsList')")
-        if (is(try(validObject(object@unlistData, complete=TRUE), silent=TRUE),
-               "try-error")) {
-            object@unlistData <- updateObject(object@unlistData)
-            return(object)
-        }
-        object
-    }
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion.
@@ -271,7 +248,7 @@ setMethod("rglist", "GAlignmentsList",
     }
 )
 
-setAs("GAlignmentsList", "Ranges", 
+setAs("GAlignmentsList", "IntegerRanges", 
     function(from) ranges(from, use.names=TRUE, use.mcols=TRUE)
 )
 setAs("GAlignmentsList", "GRanges", 
@@ -280,7 +257,7 @@ setAs("GAlignmentsList", "GRanges",
 setAs("GAlignmentsList", "GRangesList", 
     function(from) grglist(from, use.names=TRUE, use.mcols=TRUE)
 )
-setAs("GAlignmentsList", "RangesList", 
+setAs("GAlignmentsList", "IntegerRangesList", 
     function(from) rglist(from, use.names=TRUE, use.mcols=TRUE)
 )
 
@@ -311,6 +288,9 @@ setAs("GAlignmentsList", "GAlignmentPairs", function(from) {
                           names=names(ga)[first])
       })
 
+setAs("list", "GAlignmentsList", function(from) do.call(GAlignmentsList, from))
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
 ###
