@@ -57,10 +57,10 @@ encodeOverlaps1 <- function(query, subject,
                             query.break=0L, flip.query=FALSE,
                             as.matrix=FALSE, as.raw=FALSE)
 {
-    if (!is(query, "Ranges"))
-        stop("'query' must be a Ranges object")
-    if (!is(subject, "Ranges"))
-        stop("'subject' must be a Ranges object")
+    if (!is(query, "IntegerRanges"))
+        stop("'query' must be an IntegerRanges object")
+    if (!is(subject, "IntegerRanges"))
+        stop("'subject' must be an IntegerRanges object")
     if (is.numeric(query.space) && !is.integer(query.space))
         query.space <- as.integer(query.space)
     if (is.numeric(subject.space) && !is.integer(subject.space))
@@ -85,8 +85,8 @@ encodeOverlaps1 <- function(query, subject,
 
 ### TODO: Put this in the (upcoming) man page for encodeOverlaps().
 ### A simple (but inefficient) implementation of the "findOverlaps" method for
-### Ranges objects. Complexity and memory usage is M x N where M and N are the
-### lengths of 'query' and 'subject', respectively.
+### IntegerRanges objects. Complexity and memory usage is M x N where M and N
+### are the lengths of 'query' and 'subject', respectively.
 findRangesOverlaps <- function(query, subject)
 {
     ovenc <- encodeOverlaps1(query, subject, as.matrix=TRUE, as.raw=TRUE)
@@ -167,16 +167,16 @@ findRangesOverlaps <- function(query, subject)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### encodeOverlaps() generic and methods for RangesList objects.
+### encodeOverlaps() generic and methods for IntegerRangesList objects.
 ###
 
 setGeneric("encodeOverlaps", signature=c("query", "subject"),
     function(query, subject, hits=NULL, ...) standardGeneric("encodeOverlaps")
 )
 
-setMethods("encodeOverlaps", list(c("RangesList", "RangesList"),
-                                  c("RangesList", "Ranges"),
-                                  c("Ranges", "RangesList")),
+setMethods("encodeOverlaps", list(c("IntegerRangesList", "IntegerRangesList"),
+                                  c("IntegerRangesList", "IntegerRanges"),
+                                  c("IntegerRanges", "IntegerRangesList")),
     function(query, subject, hits=NULL, ...)
     {
         .RangesList_encodeOverlaps(as.list(start(query)),
@@ -758,9 +758,9 @@ extractQueryStartInTranscript <- function(query, subject,
                  "indicating for each paired-end read the position of the ",
                  "break between the ranges coming from one end and those ",
                  "coming from the other end")
-        query <- ptail(query, n=-query.break)
+        query <- tails(query, n=-query.break)
     }
-    query1 <- unlist(phead(query, n=1L), use.names=FALSE)
+    query1 <- unlist(heads(query, n=1L), use.names=FALSE)
     ## A sanity check.
     if (length(query1) != length(query)) 
         stop("some list elements in 'query' are empty")
